'Build Version: 3.0.104.0
'Build Version: 3.0.74.0
 '** ******************************** mpfcomcreateuser.vbs ************************************
 '**
 '** PURPOSE:         Generates an XML request that provisions a new user in Active Directory
 '**                  within the hosting OU on a specified preferred domain controller.
 '**
 '** USAGE:           cscript.exe mpfcomcreateuser.vbs /Upn:user@myorg.com /Policy:[Customer]
 '**                  [Reseller] /Domain:Containername /UsrName:MPFusername /GvName:Givenname
 '**                  /Surname:Usersurname /PDC:PreferredDomainController
 '**
 '** SCRIPT OUTPUT:   Displays XML request and XML response text.
 '** 
 '** REQUIREMENTS:    Run script on local MPF engine machine or a remote server.
 '**
 '** OBECTS REQUIRED: Mapsclient.dll and Msxml13.dll installed on the local system where 
 '**                  script is run. If not installed, use the following command to install:
 '**                  regsvr32.exe filename.dll
 '** 
 '** HISTORY:         Created 6/04
 '**
 '**
 '**                         Copyright 2004, Microsoft Corporation
 '**
 '** ******************************************************************************************

 OPTION EXPLICIT

 '** Declare global script variables:
  Dim strSpace
  Dim objArgs
  Dim objNamed
  Dim blnRetVal
  Dim strUpn
  Dim strPolicy
  Dim strDomain
  Dim strUsrName
  Dim strSurname
  Dim strGvName
  Dim strPDC

 '** Declare global script constants:
  CONST KNOERROR = 0
  CONST BLNFALSE = False
  CONST BLNTRUE  = True

  ON ERROR RESUME NEXT
 
 '** Main execution begins. . ."

 '** Create spacer for formatting output messages:
     If Len(Time) = 10 Then
     strSpace = "            "
     ElseIf Len(Time) = 11 Then
     strSpace = "             "
     End If

 '** PERFORM PRELIMINARY CHECKS. . .
 '** Call routine to check for WSH version 5.6:
     IsWSH56()

 '** Call routine to detect the default script host:
     IsHostCScript()

 '** CREATE ARGUMENT COLLECTIONS. . .
     Set objArgs  = WScript.Arguments
     Set objNamed = objArgs.Named

 '** CALL FUNCTION TO CHECK ARGUMENT CONFIGURATION. . .
     blnRetVal = GetArgConfig(strUpn,     _  
                              strPolicy,  _  
                              strDomain,  _
                              strUsrName, _  
                              strGvName,  _  
                              strSurname, _  
                              strPdc)

 '** CALL SUB TO CREATE XML PROVISIONING REQUEST. . .
     If blnRetVal = BLNTRUE Then
     CreateRequest()
     Else
     WScript.Echo "Invalid argument configuration, see script Help. . ."
     WScript.Echo ""
     ShowUsage()
     End If

 '** RELEASE OBJECT REFERENCES. . .
     Set objArgs  = Nothing
     Set objNamed = Nothing

 '** Main execution ends

'********************************************************************************
 Sub IsWSH56()

 '** Detects for required WSH 5.6 version. If none of the If blocks execute, WSH 
 '** version = 5.6

  Dim strWshVersion

  ON ERROR RESUME NEXT

 '** Initialize string variable to contain WSH version:
     strWshVersion = ""

 '** Get the WSH version installed on the system:
     strWshVersion = WScript.Version

 '** Check if any errors occurred (if WSH is not installed):
     If Err.Number <> KNOERROR Then  

     WScript.Echo Time & ": Error0x" & Hex(Err.Number) & " occurred unexpectedly !" & vbcrlf & _
                  strSpace & "Please check if WSH 5.6 is installed on your system. . ."

         If Err.Description > "" Then
         WScript.Echo strSpace & "Error description: " & Err.Description
         End If

     Err.Clear
     WScript.Quit(0)

 '** In case strWshVersion variable did not get set:
     ElseIf strWshVersion = "" OR strWshVersion = NULL Then

     Msgbox "Could not determine WSH version. To make sure"                    & vbcrlf & _
            "you have the required WSH version 5.6, you"                       & vbcrlf & _
            "can download it from the Microsoft scripting"                     & vbcrlf & _
            "site at: http://msdn.microsoft.com/scripting. . .", 64, "Script Host Message"

     WScript.Quit(0)

 '** If WSH is not version 5.6:
     ElseIf strWshVersion <> "5.6" Then

     Msgbox "WSH version 5.6 required ! Please download and"       & vbcrlf & _
            "install WSH version 5.6 from the Microsoft scripting" & vbcrlf & _
            "site before running this script. . ."        & vbcrlf & vbcrlf & _
            "http://msdn.microsoft.com/scripting", 64, "Script Host Message"

     WScript.Quit(0)
     End If

 End Sub

 '********************************************************************************
 Sub IsHostCScript()
 
 '** Detects if WSH default host is cscript.exe. If none of the If blocks execute, 
 '** host is cscript.exe. . .

  Dim strHostPath
  Dim intRetVal

  CONST KRETVAL_YES = 6
  CONST KRETVAL_NO  = 7

  ON ERROR RESUME NEXT

 '** Initialize string variable:
     strHostPath = ""

 '** Get default script host name:
     strHostPath = LCase(WScript.FullName)

 '** Check if any errors occurred:
     If Err.Number <> KNOERROR Then

     intRetVal = Msgbox _
     ("The default WSH script host was not detected. . ."       & vbcrlf & _
      "Error0x" & Hex(Err.Number) & " occurred unexpectedly !"  & vbcrlf & _
      "Error description: " & Err.Description & vbcrlf          & vbcrlf & _
      "If you continue, the script might fail to execute. . "   & vbcrlf & _
      "Do you want to continue anyway ?", 20, "Script Host Error")

           If intRetVal = KRETVAL_YES Then
       '** Do nothing and proceed with script execution...
           Err.Clear

           ElseIf intRetVal = KRETVAL_NO Then
           Err.Clear
           WScript.Quit(0)
           End If

 '** In case strHostPath variable was not set by WScript.FullName:
     ElseIf strHostPath = "" OR strHostPath = NULL Then

     Msgbox "Could not determine default script host, the script may behave" & vbcrlf & _
            "unexpectedly. . .", 64, "Script Host Message"

 '** If default script host is set to WScript or if user happens to run the script with 
 '** WScript.exe at the command line, then send a message to the user to change the 
 '** default host:
     ElseIf InStr(strHostPath, "cscript.exe") = 0 Then

     Msgbox "Script requires CSCRIPT.EXE to execute !"          & vbcrlf & _
            "Either specify cscript.exe in the command line"    & vbcrlf & _
            "or change the default script host to cscript.exe"  & vbcrlf & _
            "by running the following command:"        & vbcrlf & vbcrlf & _
            "cscript.exe //H:cscript //S", 64, "Script Host Message"

     WScript.Quit(0)
     End If

 End Sub

 '********************************************************************************
 Sub CreateRequest()

 '** Creates an XML request and submits it to MPF engine using the MPS Client COM 
 '** Object. . .

  Dim objXML
  Dim objMPFClient
  Dim strXMLCreateUser
  Dim objOrg
  Dim strXMLResponse

  ON ERROR RESUME NEXT

 '** CREATE OBJECTS REQUIRED. . .

 '** CREATE XML DOCUMENT OBJECT; IF NOT ON SYSTEM, ABORT. . .
     Set objXML = CreateObject("MSXML2.DOMDocument")

 '** Verify that XML Document object was instantiated:  
     If Not IsObject(objXML) Then

         If Err.Number <> KNOERROR Then
         WScript.Echo Time & ": Error0x" & Hex(Err.Number) & " occurred creating the XML document automation object !" & vbcrlf & _
                      strSpace & "Please make sure that Msxml13.dll is installed on your system. . ."

             If Err.Description <> "" Then
             WScript.Echo strSpace & "Description: " & Err.Description
             End If

         Err.Clear

         Else
         WScript.Echo Time & ": The XML document object could not be created ! Please make sure that Msxml13.dll" & vbcrlf & _
                      strSpace & "is installed on your system. . ."
         End If

     WScript.Quit(0)
     End If

 '** CREATE MPF COM CLIENT; IF NOT ON SYSTEM, ABORT. . .
 '** Set objMPFClient = CreateObject("Provisioning.ProvEngineClient")

 '** Verify that MPF COM Client object was instantiated:
     If Not IsObject(objMPFClient) Then

         If Err.Number <> KNOERROR Then
         WScript.Echo Time & ": Error0x" & Hex(Err.Number) & " occurred creating the MPF Client COM object !" & vbcrlf & _
                      strSpace & "Please make sure that Mapsclient.dll is installed on your system. . ."

             If Err.Description <> "" Then
             WScript.Echo strSpace & "Description: " & Err.Description
             End If

         Err.Clear

         Else
         WScript.Echo Time & ": The MPF Client COM object could not be created ! Please make sure that" & vbcrlf & _
                      strSpace & "Mapsclient.dll is installed on your system. . ."
         End If

     WScript.Quit(0)
     End If
  
 '** DEFINE THE XML REQUEST STRING. . .
     strXMLCreateUser = "<?xml version=""1.0"" encoding=""utf-8"" ?>"                                                          & _
                        "<request>"                                                                                            & _
	                    "<data>"                                                                                           & _
		                "<userPrincipalName/>"                                                                         & _
		                "<policyName/>"                                                                                & _
                                "<container/>"                                                                                 & _
		                "<displayName/>"                                                                               & _
		                "<givenName/>"                                                                                 & _
		                "<sn/>"                                                                                        & _
		                "<preferredDomainController/>"                                                                 & _
	                    "</data>"                                                                                          & _
	                    "<procedure>"                                                                                      & _
		                "<execute namespace=""Managed Active Directory"" procedure=""CreateUser"">"                    & _
			            "<before source=""data"" destination=""executeData"" mode=""merge""/>"                     & _
			            "<after source=""executeData"" sourcePath=""user"" destination=""data"" mode=""insert""/>" & _
		                "</execute>"                                                                                   & _
	                    "</procedure>"                                                                                     & _
                        "</request>"

 '** Load an XML Template that provides the basis for creating the XML request:
     objXml.loadXML strXMLCreateUser

     If Err.Number <> KNOERROR Then
     WScript.Echo Time & ": Error0x" & Hex(Err.Number) & " occurred loading the XML document request template !"
        
         If Err.Description <> "" Then
         WScript.Echo strSpace & "Description: " & Err.Description
         End If
        
     Err.Clear
     WScript.Quit(0)
     End If
    
 '** FILL IN THE REQUEST PROPERTIES IN THE XML FILE. . .
 '** Set the user principal name:
     Set objOrg = objXml.SelectSingleNode("//*/data/userPrincipalName")
     objOrg.Text = strUpn 

 '** Set the policy type:
 '** this can be either "customer" or "reseller"
     Set objOrg = objXml.SelectSingleNode("//*/data/policyName")
     objOrg.Text = strPolicy

 '** Set the domain component name and the OU container to create the user in:
     Set objOrg = objXml.SelectSingleNode("//*/data/container")
     objOrg.Text = "LDAP://OU=myOrg,OU=reseller,OU=Hosting,DC=" & strDomain & ",DC=Com"

 '** Set the display name:
     Set objOrg = objXml.SelectSingleNode("//*/data/displayName")
     objOrg.Text = strUsrName
	
 '** Set the given name:
     Set objOrg = objXml.SelectSingleNode("//*/data/givenName")
     objOrg.Text = strGvName

 '** Set the user surname:
     Set objOrg = objXml.SelectSingleNode("//*/data/sn")
     objOrg.Text = strSurname

 '** Set the preferred domain controller on which the request will be performed. . .
     Set objOrg = objXml.SelectSingleNode("//*/data/preferredDomainController")
     objOrg.Text = strPDC

 '** DISPLAY THE XML REQUEST TEXT. . .
     WScript.Echo Time & ": XML request text = " & vbcrlf & vbcrlf & _
                            objXML.XML

 '** SUBMIT THE XML REQUEST. . .
     strXMLResponse = objMPFClient.SubmitRequest(objXml.Xml, "Managed Active Directory","CreateUser")

     '** Check for errors on the network and/or the SubmitRequest method:
         If Err.Number <> KNOERROR Then
         WScript.Echo Time & ": Error0x" & Hex(Err.Number) & " occurred; the XML provisioning request could not be processed !"

         If Err.Description <> "" Then
         WScript.Echo strSpace & "Description: " & Err.Description & ". . ."
         End If
        
         Err.Clear
         WScript.Quit(0)
         End If

 '** DISPLAY THE RESPONSE. . .
     If strXMLResponse <> "" Then
     WScript.Echo Time & ": Return value of SubmitRequest method = " & vbcrlf & vbcrlf & _
                            strXMLResponse
     Else
     WScript.Echo Time & ": A response from the MPF engine could not be retrieved, please try again. . ."
     End If

 '** RELEASE OBJECT REFERENCES. . .
     Set objOrg       = Nothing
     Set objMPFClient = Nothing
     Set objXML       = Nothing

 End Sub

 '********************************************************************************
 Function GetArgConfig(ByRef strUpn,     _
                       ByRef strPolicy,  _
                       ByRef strDomain,  _
                       ByRef strUsrName, _
                       ByRef strGvName,  _
                       ByRef strSurname, _
                       ByRef strPdc)

  ON ERROR RESUME NEXT
 
 '** SET FUNCTION RETURN VALUE TO TRUE. . .
 '** Will get set to BLNFALSE if an invalid argument configuration is detected:

     GetArgConfig = BLNTRUE

 '** If no args are supplied:

     If objArgs.Count = 0 Then
     GetArgConfig = BLNFALSE
     Exit Function    

 '** Check if user is asking for Help:

     ElseIf objArgs.Count = 1 Then

         If objArgs.Item(0) = "?"     OR _
            objArgs.Item(0) = "/?"    OR _
            objArgs.Item(0) = "/h"    OR _
            objArgs.Item(0) = "/H"    OR _
            objArgs.Item(0) = "-h"    OR _
            objArgs.Item(0) = "-H"    OR _
            objArgs.Item(0) = "-help" OR _
            objArgs.Item(0) = "-Help" OR _
            objArgs.Item(0) = "help"  OR _
            objArgs.Item(0) = "Help"  OR _
            objArgs.Item(0) = "/help" OR _
            objArgs.Item(0) = "/Help" Then

         ShowUsage()
         WScript.Quit()
         Exit Function

         Else
         GetArgConfig = BLNFALSE
         Exit Function
         End If

 '** Otherwise, check for valid argument configuration:

     ElseIf objArgs.Count = 7 Then

         If objNamed.Exists("Upn") Then
         strUpn = LCase(objNamed("Upn"))
         Else
         GetArgConfig = BLNFALSE
         Exit Function
         End If

         If objNamed.Exists("Policy") Then
         strPolicy = LCase(objNamed("Policy"))
         Else
         GetArgConfig = BLNFALSE
         Exit Function
         End If

         If objNamed.Exists("Domain") Then
         strDomain = LCase(objNamed("Domain"))
         Else
         GetArgConfig = BLNFALSE
         Exit Function
         End If

         If objNamed.Exists("UsrName") Then
         strUsrName = LCase(objNamed("UsrName"))
         Else
         GetArgConfig = BLNFALSE
         Exit Function
         End If

         If objNamed.Exists("GvName") Then
         strGvName = LCase(objNamed("GvName"))
         Else
         GetArgConfig = BLNFALSE
         Exit Function
         End If

         If objNamed.Exists("Surname") Then
         strSurname = LCase(objNamed("Surname"))
         Else
         GetArgConfig = BLNFALSE
         Exit Function
         End If

         If objNamed.Exists("PDC") Then
         strPDC = LCase(objNamed("PDC"))
         Else
         GetArgConfig = BLNFALSE
         Exit Function
         End If

     Else
     GetArgConfig = BLNFALSE
     End If

 '** Check for unexpected errors:
     If Err.Number <> KNOERROR Then

     WScript.Echo Time & ": Error0x" & Hex(Err.Number) & " occurred unexpectedly when parsing arguments. . ."

         If Err.Description <> "" Then
         WScript.Echo strSpace & "Description: " & Err.Description
         End If

     Err.Clear
     WScript.Quit(0)
     End If

 End Function

 '********************************************************************************
 Sub ShowUsage()

 WScript.Echo "********************** mpfcomCreateuser.vbs ***************************"
 WScript.Echo ""
 WScript.Echo "SCRIPT HELP. . ."
 WScript.Echo ""
 WScript.Echo "PURPOSE: Generates an XML request for creating an Active Directory"
 WScript.Echo "         user account and submits it to the MPF provisioning engine. ."
 WScript.Echo ""
 WScript.Echo "NAMED ARGUMENTS:"
 WScript.Echo "/Upn:Value       Value specifies the user principal name."
 WScript.Echo "/Policy:Value    Value specifies policy type as ""customer"" or ""reseller."""
 WScript.Echo "/Domain:Value    Value specifies the domain container in which to create"
 WScript.Echo "                 user, such as ""fabrikam."""
 WScript.Echo "/UsrName:Value   Value specifies the MPF user display name."
 WScript.Echo "/GvName:Value    Value specifies the user given name."
 WScript.Echo "/Surname:Value   Value specifies the user surname."
 WScript.Echo "/PDC:Value       Value specifies the preferred domain controller, such"
 WScript.Echo "                 as ""AD01.fabrikam.com."""
 WScript.Echo "/?               Displays these script usage instructions."
 WScript.Echo ""
 WScript.Echo "USAGE EXAMPLE:"
 WScript.Echo "cscript.exe mpfcomcreateuser.vbs /Upn:user@myorg.com /Policy:[Customer][Reseller]"
 WScript.Echo "/Domain:Containername /UsrName:MPFusername /GvName:Givenname /Surname:Usersurname"
 WScript.Echo "/PDC:PreferredDomainController"
 WScript.Echo ""
 WScript.Echo "NOTES:"
 WScript.Echo "1. Run script locally on the MPF engine machine or on a remote server."
 WScript.Echo "2. Script requires WSH version 5.6."
 WScript.Echo "3. Mapsclient.dll and Msxml13.dll must be registered on your local system."
 WScript.Echo "   If not on system, register with RegSvr32.exe."
 WScript.Echo "4. Use quotes for any named arguments that contain spaces."

 End Sub







